﻿Function Get-dvnDCService
{
<#
.NOTES
    Author: Mason Willinger
    Version: 20160229.1


    Version History

    Version: 20160229.1
        Original code creation

.SYNOPSIS
    Get a service or services running on all domain controllers in one or more Active Directory domains or forests.
.DESCRIPTION
    This function will attempt to get the status of a service or services running on all domain controllers in user-specified AD domains or forests.
.PARAMETER Name
    The name(s) of the service(s) that you would like to get.
.PARAMETER Domain
    The name(s) of the Active Directory domain(s) where you would like to search for instances of the specified service(s) configured/running on domain controllers. If no value is specified for this parameter or the Forest paramenter, this parameter will be set to the user's current domain.
.PARAMETER Forest
    The name(s) of the Active Directory forest(s) where you would like to search for instances of the specified service(s) configured/running on domain controllers.

.EXAMPLE
    Get-dvnDCService wincollect
    Attempts to get the service named "wincollect" from all domain controllers in the user's current Active Directory domain.
.EXAMPLE
    Get-dvnDCService -Forest dvn.dmz -Name "wincollect","ccmexec"
    Attempts to get the services named "wincollect" and "ccmexec" from all domain controllers in the Active Directory forest dvn.dmz.
.EXAMPLE
    Get-dvnDCService -Domain "corp.dvn.com","partners.dvn.dmz" DNS
    Attempts to get the service named "DNS" from all domain controllers in the Active Directory domains corp.dvn.com and partners.dvn.dmz.
#>

    [CmdletBinding()]
    Param
    (
        [Parameter(Mandatory=$true,
                   Position=0,
                   ValueFromPipeline=$true)]
        [String[]]$Name,
        [Parameter(Mandatory=$false,
                   Position=1)]
        [ValidateScript(
        {
            #Verify that each specified domain name is valid.
            try
            {
                foreach($domainName in $_)
                {
                    [ADSI]::Exists("LDAP://" + $domainName)
                }
            }
            catch
            {
                Throw "`nError validating domain name. The AD domain $domainName could not be contacted."
            }
        })]
        [String[]]$Domain,
        [Parameter(Mandatory=$false,
                   Position=2)]
        [ValidateScript(
        {
            #Verify that each specified forest name is valid.
            try
            {
                foreach($forestName in $_)
                {
                    [ADSI]::Exists("LDAP://" + $forestName)
                }
            }
            catch
            {
                Throw "`nError validating forest name. The AD forest $forestName could not be contacted."
            }
        })]
        [String[]]$Forest
    )

    Begin
    {
        $DCs = $null

        if($Domain)
        {
            #If the Domain parameter is used, get all domain controllers from all domains specified. The Forest parameter will not be checked if the Domain parameter contains valid AD domain names.
            foreach ($adDomain in $Domain)
            {
                $DCs += (Get-ADDomain $adDomain).replicaDirectoryServers
            }
        }
        else
        {
            #If the Domain parameter is empty, and the Forest parameter is used, get all domain controllers from all forests specified.
            if($Forest)
            {
                foreach ($adForest in $Forest)
                {
                    $DCs += (Get-ADForest $adForest).globalCatalogs
                }
            }
            #If both the Domain and Forest parameters are empty, use the user's current domain.
            else
            {
                $DCs = (Get-ADDomain $env:USERDOMAIN).replicaDirectoryServers
            }
        }
    }

    Process
    {
        #Attempt to get all service names specified from all domain controllers that reside in all specified domains or forests.
        foreach ($service in $Name)
        {
            foreach ($DC in $DCs)
            {
                Get-WmiObject -Class Win32_Service -ComputerName $DC -Filter "Name = '$service'" | select PSComputerName,Name,State,Started,StartMode,StartName
            }
        }
    }
}